package net.idware.android.idwaretest;

import java.lang.ref.WeakReference;

import net.idware.android.dlplib.DriverLicense;
import net.idware.android.dlplib.IDriverLicenseParser;
import net.idware.android.dlplib.ParserException;
import net.idware.android.dlplib.DriverLicenseParserFactory;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;
import android.app.Activity;

/**
 * Here is a simple example how you can use IDScan.net 
 * parsing web services.
 * 
 * @author andrew0x1@gmail.com
 */
public class MainActivity extends Activity
{
	/**
	 * Parser's Results.
	 * 
	 * @author andrew0x1@gmail.com
	 */
	private interface IResultHandler
	{
		void onDriverLicense(DriverLicense result);
		void onError(ParserException error);
	}
	
	/**
	 * We have to call our web services. In our case we will do in in 
	 * a separate thread. We will use AsyncTask 
	 * 
	 * @author andrew0x1@gmail.com
	 */
	private final static class _ParsingAction extends AsyncTask<String, Integer, DriverLicense>
	{
		private final WeakReference<IResultHandler> _handler;
		private volatile ParserException _err = null;
		
		_ParsingAction(IResultHandler handler)
		{
			if(handler == null)
				throw new NullPointerException("Parameter 'handler' can't be null.");
			
			_handler = new WeakReference<IResultHandler>(handler);
		}
		
		@Override
		protected DriverLicense doInBackground(String... params)
		{
			try
			{
				//We need to create an instance for parser and create 
				//a web service call.In order to do that we need to 
				// add <uses-permission android:name="android.permission.INTERNET" /> 
				// in AndroidManifest.xml.
				IDriverLicenseParser parser = DriverLicenseParserFactory.createPaser("<enter the authkey here>");
				return parser.parse(params[0]);
			}
			catch(ParserException ex)
			{
				_err = ex;
				return null;
			}
		}
		
		@Override
		protected void onPostExecute(DriverLicense result)
		{
			//Processing results in the main UI thread .
			IResultHandler h = _handler.get();
			if(h != null)
			{
				if(result != null)
					h.onDriverLicense(result);
				else if(_err != null)
					h.onError(_err);
			}
		}
	}
	
	private Button _btn_parse; 
	private EditText _et_input;
	private TextView _tv_output;
	
	/** Current async parsing. */
	private _ParsingAction _current_action = null;
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		
		_btn_parse = (Button)findViewById(R.id.btn_parse);
		_et_input = (EditText)findViewById(R.id.et_input);
		_tv_output = (TextView)findViewById(R.id.tv_output);
		
		if(savedInstanceState == null)
		{
			/** Insert your test data here. */
			String text = "@\n\nAAMVA6360510101DL00310219DLDAQ022356834\nDAASMITH,JOHN,\nDAYBRO\nDAG\nDAI\nDAJMS\nDAK           \nDARE\nDASB         \nDATY    \nDAU501\nDAW170\nDBA2005121\n2\nDBB19841229\nDBCM\nDBD20001229\nDAL750 SAMPLE ST.\nDANLILBURN\nDAOMS\nDAP30047      SAMPLE ST.\n\n";
			_et_input.setText(text);
		}
		
		_btn_parse.setOnClickListener(new OnClickListener()
		{
			@Override
			public void onClick(View v)
			{
				_parse();
			}
		});
	}
	
	@Override
	protected void onDestroy()
	{
		super.onDestroy();
		_cancel();
	}
	
	@Override
	protected void onResume()
	{
		super.onResume();
		_updateUI();
	}
	
	private final void _parse()
	{
		final String data = _et_input.getText().toString();
		if(data.length() == 0)
		{
			Toast.makeText(this, "Input field is empty!", Toast.LENGTH_SHORT).show();
			return;
		}
		if(_current_action == null)
		{
			_tv_output.setText(R.string.parsing);
			
			_current_action = new _ParsingAction(_handler);
			_updateUI();
			_current_action.execute(data);
		}
	}
	
	private final void _cancel()
	{
		if(_current_action != null)
		{
			_current_action.cancel(false);
			_current_action = null;
			_updateUI();
		}
	}
	
	private final void _updateUI()
	{
		if(_current_action == null)
		{
			_btn_parse.setEnabled(true);
		}
		else
		{
			_btn_parse.setEnabled(false);
		}
	}
	
	private final IResultHandler _handler = new IResultHandler()
	{
		@Override
		public void onDriverLicense(DriverLicense result)
		{
			_current_action = null;
			_updateUI();
			
			final String address = result.getAddress1();
			if(address != null)
				_tv_output.setText("Address: " + address);
			else
				_tv_output.setText("Address: empty");
		}

		@Override
		public void onError(ParserException error)
		{
			_current_action = null;
			_updateUI();
			
			_tv_output.setText("Error");
			
			final String msg = error.getMessage();
			if(msg != null)
				Toast.makeText(MainActivity.this, msg, Toast.LENGTH_SHORT).show();
		}
	};
}
